@@ -29,6 +29,7 @@ import locale
 import pango
 from series import SeriesManager, Show, Episode
 from lib import constants
+from settings import Settings
 
 _ = gettext.gettext
 
@@ -38,12 +39,31 @@ class MainWindow(hildon.Window):
     
     def __init__(self):
         super(MainWindow, self).__init__()
+        
+        # i18n
+        languages = []
+        lc, encoding = locale.getdefaultlocale()
+        if lc:
+            languages = [lc]
+        languages += constants.DEFAULT_LANGUAGES
+        gettext.bindtextdomain(constants.SF_COMPACT_NAME,
+                               constants.LOCALE_DIR)
+        gettext.textdomain(constants.SF_COMPACT_NAME)
+        language = gettext.translation(constants.SF_COMPACT_NAME,
+                                       constants.LOCALE_DIR,
+                                       languages = languages,
+                                       fallback = True)
+        _ = language.gettext
+        
         self.series_manager = SeriesManager()
-        self.series_manager.load(constants.SF_CONF_FILE)
+        self.series_manager.load(constants.SF_DB_FILE)
         self.series_manager.connect('show-list-changed',
                                     self._show_list_changed_cb)
         self.series_manager.connect('get-full-show-complete',
                                     self._get_show_complete_cb)
+        
+        self.settings = Settings()
+        self.settings.load(constants.SF_CONF_FILE)
 
         self.shows_view = ShowsSelectView()
         self.shows_view.connect('row-activated', self._row_activated_cb)
@@ -117,7 +137,7 @@ class MainWindow(hildon.Window):
     
     def _row_activated_cb(self, view, path, column):
         show = self.shows_view.get_show_from_path(path)
-        seasons_view = SeasonsView(self.series_manager, show)
+        seasons_view = SeasonsView(self.settings, self.series_manager, show)
         seasons_view.connect('delete-event',
                      lambda w, e:
                         self.shows_view.set_shows(self.series_manager.series_list))
@@ -138,7 +158,8 @@ class MainWindow(hildon.Window):
         new_show_dialog.destroy()
     
     def _exit_cb(self, window, event):
-        self.series_manager.save(constants.SF_CONF_FILE)
+        self.series_manager.save(constants.SF_DB_FILE)
+        self.settings.save(constants.SF_CONF_FILE)
         gtk.main_quit()
 
     def _show_list_changed_cb(self, series_manager):
@@ -154,10 +175,15 @@ class MainWindow(hildon.Window):
 
 class ShowsSelectView(gtk.TreeView):
     
+    SHOW_NAME_COLUMN = 0
+    SHOW_OBJECT_COLUMN = 1
+    
     def __init__(self):
         super(ShowsSelectView, self).__init__()
         model = gtk.ListStore(str, gobject.TYPE_PYOBJECT)
-        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text = 0)
+        show_renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn('Name', show_renderer, text = self.SHOW_NAME_COLUMN)
+        column.set_cell_data_func(show_renderer, self._show_select_view_data_func)
         self.set_model(model)
         self.append_column(column)
         self.sort_ascending()
@@ -166,24 +192,58 @@ class ShowsSelectView(gtk.TreeView):
         model = self.get_model()
         model.clear()
         for show in shows:
-            model.append([show.name, show])
+            row = {self.SHOW_NAME_COLUMN: show.name,
+                   self.SHOW_OBJECT_COLUMN: show
+                  }
+            model.append(row.values())
 
     def get_show_from_path(self, path):
         model = self.get_model()
-        return model[path][1]
+        return model[path][self.SHOW_OBJECT_COLUMN]
     
     def sort_descending(self):
-        self.get_model().set_sort_column_id(0, gtk.SORT_DESCENDING)
+        self.get_model().set_sort_column_id(self.SHOW_NAME_COLUMN, gtk.SORT_DESCENDING)
     
     def sort_ascending(self):
-        self.get_model().set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.get_model().set_sort_column_id(self.SHOW_NAME_COLUMN, gtk.SORT_ASCENDING)
+    
+    def _show_select_view_data_func(self, column, renderer, model, iter):
+        show = model.get_value(iter, self.SHOW_OBJECT_COLUMN)
+        if not show:
+            return
+        seasons = len(show.get_seasons())
+        if seasons:
+            show_info = '<small><span foreground="%s">' % constants.SECONDARY_TEXT_COLOR
+            show_info += gettext.ngettext('%s season', '%s seasons', seasons) \
+                         % seasons
+            if show.is_completely_watched():
+                show_info += ' | ' + _('Completely watched')
+            else:
+                episodes_to_watch = len([episode for episode in show.episode_list \
+                                    if not episode.watched])
+                if episodes_to_watch:
+                    show_info += ' | ' + gettext.ngettext('%s episode not watched',
+                                                          '%s episodes not watched',
+                                                          episodes_to_watch) \
+                                                          % episodes_to_watch
+                else:
+                    show_info += ' | ' + _('No episodes to watch')
+            show_info += '</span></small>'
+        else:
+            show_info = ''
+        renderer.set_property('markup',
+                              '<b>%s</b>\n' % show.name +
+                              show_info)
+        renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
 
 class SeasonsView(hildon.Window):
     
-    def __init__(self, series_manager, show):
+    def __init__(self, settings, series_manager, show):
         super(SeasonsView, self).__init__()
         self.set_title(show.name)
         
+        self.settings = settings
+        
         self.series_manager = series_manager
         self.series_manager.connect('update-show-episodes-complete',
                                     self._update_show_episodes_complete_cb)
@@ -191,7 +251,7 @@ class SeasonsView(hildon.Window):
         self.set_menu(self._create_menu())
         self.set_title(show.name)
 
-        self.seasons_select_view = SeasonSelectView()
+        self.seasons_select_view = SeasonSelectView(self.show)
         seasons = self.show.get_seasons()
         self.seasons_select_view.set_seasons(seasons)
         self.seasons_select_view.connect('row-activated', self._row_activated_cb)
@@ -203,7 +263,7 @@ class SeasonsView(hildon.Window):
     
     def _row_activated_cb(self, view, path, column):
         season = self.seasons_select_view.get_season_from_path(path)
-        episodes_view = EpisodesView(self.show, season)
+        episodes_view = EpisodesView(self.settings, self.show, season)
         episodes_view.connect('delete-event', self._update_series_list_cb)
         episodes_view.connect('episode-list-changed', self._update_series_list_cb)
         episodes_view.show_all()
@@ -220,7 +280,6 @@ class SeasonsView(hildon.Window):
         menu.append(menuitem)
         
         menuitem = gtk.MenuItem(_('Edit Info'))
-        menuitem.set_sensitive(False)
         menuitem.connect('activate', self._edit_show_info)
         menu.append(menuitem)
         
@@ -230,7 +289,6 @@ class SeasonsView(hildon.Window):
             menu.append(menuitem)
         
         menuitem = gtk.MenuItem(_('New Episode'))
-        menuitem.set_sensitive(False)
         menuitem.connect('activate', self._new_episode_cb)
         menu.append(menuitem)
         
@@ -250,7 +308,7 @@ class SeasonsView(hildon.Window):
         infotextview = InfoTextView()
         infotextview.set_title(self.show.name)
         infotextview.add_field (self.show.overview)
-        infotextview.add_field (_('\n'))
+        infotextview.add_field ('\n')
         infotextview.add_field (self.show.genre, _('Genre'))
         infotextview.add_field (self.show.network, _('Network'))
         infotextview.add_field (self.show.actors, _('Actors'))
@@ -316,10 +374,17 @@ class SeasonsView(hildon.Window):
     
 class SeasonSelectView(gtk.TreeView):
     
-    def __init__(self):
+    SEASON_NAME_COLUMN = 0
+    SEASON_NUMBER_COLUMN = 1
+    
+    def __init__(self, show):
         super(SeasonSelectView, self).__init__()
+        self.show = show
         model = gtk.ListStore(str, str)
-        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text = 0)
+        season_renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn('Name', season_renderer, text = 0)
+        column.set_cell_data_func(season_renderer,
+                                  self._season_select_view_data_func)
         self.set_model(model)
         self.append_column(column)
 
@@ -331,39 +396,81 @@ class SeasonSelectView(gtk.TreeView):
                 name = _('Special')
             else:
                 name = _('Season %s') % season
-            model.append([name, season])
+            row = {self.SEASON_NAME_COLUMN: name,
+                   self.SEASON_NUMBER_COLUMN: season,
+                  }
+            model.append(row.values())
     
     def get_season_from_path(self, path):
         model = self.get_model()
         iter = model.get_iter(path)
-        season = model.get_value(iter, 1)
+        season = model.get_value(iter, self.SEASON_NUMBER_COLUMN)
         return season
+    
+    def _season_select_view_data_func(self, column, renderer, model, iter):
+        season = model.get_value(iter, self.SEASON_NUMBER_COLUMN)
+        if season == '0':
+            name = _('Special')
+        else:
+            name = _('Season %s') % season
+        episodes = self.show.get_episode_list_by_season(season)
+        episodes_to_watch = [episode for episode in episodes \
+                            if not episode.watched]
+        season_info = ''
+        if not episodes_to_watch:
+            if episodes:
+                name = '<span foreground="%s">%s</span>' % \
+                        (constants.SECONDARY_TEXT_COLOR, name)
+                season_info = _('Completely watched')
+        else:
+            number_episodes_to_watch = len(episodes_to_watch)
+            season_info = gettext.ngettext('%s episode not watched',
+                                           '%s episodes not watched',
+                                           number_episodes_to_watch) \
+                                           % number_episodes_to_watch
+            sorted_episodes_to_watch = [episode.episode_number for episode in \
+                                        episodes_to_watch]
+            sorted_episodes_to_watch.sort()
+            next_episode = None
+            for episode in episodes_to_watch:
+                if episode.episode_number == sorted_episodes_to_watch[0]:
+                    next_episode = episode
+                    break
+            season_info += ' | ' + _('<i>Next to watch:</i> %s') % episode
+        renderer.set_property('markup',
+                              '<b>%s</b>\n'
+                              '<span foreground="%s">%s</span>' % \
+                              (name, constants.SECONDARY_TEXT_COLOR, season_info))
+        renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
 
 class NewShowDialog(gtk.Dialog):
     
     def __init__(self, parent):
         super(NewShowDialog, self).__init__(parent = parent,
-                                             buttons = (gtk.STOCK_ADD,
-                                                        gtk.RESPONSE_ACCEPT))
+                                            buttons = (gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT,
+                                                       gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
         
         self.set_title(_('Edit Show'))
-        
+        self.set_default_size(600, -1);
         self.show_name = gtk.Entry()
-        self.show_overview = hildon.TextView()
-        self.show_overview.set_placeholder(_('Overview'))
+        self.show_overview = gtk.TextView()
         self.show_overview.set_wrap_mode(gtk.WRAP_WORD)
         self.show_genre = gtk.Entry()
         self.show_network = gtk.Entry()
         self.show_rating = gtk.Entry()
         self.show_actors = gtk.Entry()
         
+        winscroll = gtk.ScrolledWindow()
+        winscroll.add(self.show_overview)
+        winscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+
         contents = gtk.VBox(False, 0)
         
         row = gtk.HBox(False, 12)
         row.pack_start(gtk.Label(_('Name:')), False, False, 0)
         row.pack_start(self.show_name, True, True, 0)
         contents.pack_start(row, False, False, 0)
-        contents.pack_start(self.show_overview, False, False, 0)
+        contents.pack_start(winscroll, False, False, 0)
         
         fields = [(_('Genre:'), self.show_genre),
                   (_('Network:'), self.show_network),
@@ -378,12 +485,8 @@ class NewShowDialog(gtk.Dialog):
             row.pack_start(label, False, False, 0)
             row.pack_start(widget, True, True, 0)
             contents.pack_start(row, False, False, 0)
-        
-        contents_area = hildon.PannableArea()
-        contents_area.add_with_viewport(contents)
-        contents_area.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
-        
-        self.vbox.add(contents_area)
+                
+        self.vbox.add(contents)
         self.vbox.show_all()
     
     def get_info(self):
@@ -415,37 +518,35 @@ class NewEpisodeDialog(gtk.Dialog):
     
     def __init__(self, parent, show):
         super(NewEpisodeDialog, self).__init__(parent = parent,
-                                               buttons = (gtk.STOCK_ADD,
-                                                          gtk.RESPONSE_ACCEPT))
+                                               buttons = (gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT,
+                                                          gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
         
         self.set_title(_('New Episode'))
-        
+        self.set_default_size(600, 400);
+
         self.episode_name = gtk.Entry()
-        self.episode_overview = hildon.TextView()
-        self.episode_overview.set_placeholder(_('Overview'))
+        self.episode_overview = gtk.TextView()
         self.episode_overview.set_wrap_mode(gtk.WRAP_WORD)
         
-        self.episode_number = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
-                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        selector = hildon.TouchSelectorEntry(text = True)
+        winscroll = gtk.ScrolledWindow()
+        winscroll.add(self.episode_overview)
+        winscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+
+        self.episode_number = gtk.combo_box_entry_new_text()
         self.episode_number.set_title(_('Number:'))
         for i in xrange(20):
-            selector.append_text(str(i + 1))
-        self.episode_number.set_selector(selector)
+            self.episode_number.append_text(str(i + 1))
         self.episode_number.set_active(0)
-        
-        self.episode_season = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
-                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        selector = hildon.TouchSelectorEntry(text = True)
+
+        self.episode_season = gtk.combo_box_entry_new_text()
         self.episode_season.set_title(_('Season:'))
         seasons = show.get_seasons()
         for season in seasons:
-            selector.append_text(season)
-        self.episode_season.set_selector(selector)
+            self.episode_season.append_text(season)
         if seasons:
             self.episode_season.set_active(len(seasons) - 1)
         else:
-            selector.append_text('1')
+            self.episode_season.append_text('1')
             self.episode_season.set_active(0)
         
         self.episode_director = gtk.Entry()
@@ -459,7 +560,7 @@ class NewEpisodeDialog(gtk.Dialog):
         row.pack_start(gtk.Label(_('Name:')), False, False, 0)
         row.pack_start(self.episode_name, True, True, 0)
         contents.pack_start(row, False, False, 0)
-        contents.pack_start(self.episode_overview, False, False, 0)
+        contents.pack_start(winscroll, False, False, 0)
         row = gtk.HBox(False, 12)
         row.add(self.episode_season)
         row.add(self.episode_number)
@@ -479,11 +580,7 @@ class NewEpisodeDialog(gtk.Dialog):
             row.pack_start(widget, True, True, 0)
             contents.pack_start(row, False, False, 0)
         
-        contents_area = hildon.PannableArea()
-        contents_area.add_with_viewport(contents)
-        contents_area.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
-        
-        self.vbox.add(contents_area)
+        self.vbox.add(contents)
         self.vbox.show_all()
     
     def get_info(self):
@@ -493,8 +590,8 @@ class NewEpisodeDialog(gtk.Dialog):
         overview_text = buffer.get_text(start_iter, end_iter)
         info = {'name': self.episode_name.get_text(),
                 'overview': overview_text,
-                'season': self.episode_season.get_selector().get_entry().get_text(),
-                'number': self.episode_number.get_selector().get_entry().get_text(),
+                'season': self.episode_season.child.get_text(),
+                'number': self.episode_number.child.get_text(),
                 'director': self.episode_director.get_text(),
                 'writer': self.episode_writer.get_text(),
                 'rating': self.episode_rating.get_text(),
@@ -508,8 +605,8 @@ class EditEpisodeDialog(NewEpisodeDialog):
         
         self.episode_name.set_text(episode.name)
         self.episode_overview.get_buffer().set_text(episode.overview)
-        self.episode_season.get_selector().get_entry().set_text(episode.season_number)
-        self.episode_number.get_selector().get_entry().set_text(str(episode.episode_number))
+        self.episode_season.child.set_text(episode.season_number)
+        self.episode_number.child.set_text(str(episode.episode_number))
         self.episode_director.set_text(episode.director)
         self.episode_writer.set_text(str(episode.writer))
         self.episode_rating.set_text(episode.rating)
@@ -524,9 +621,11 @@ class EpisodesView(hildon.Window):
                                                    ()),
                    }
     
-    def __init__(self, show, season_number = None):
+    def __init__(self, settings, show, season_number = None):
         super(EpisodesView, self).__init__()
         
+        self.settings = settings
+        
         self.show = show
         self.season_number = season_number
         self.set_title(self.show.name)
@@ -542,18 +641,26 @@ class EpisodesView(hildon.Window):
         winscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.add(winscroll)
         self.set_menu(self._create_menu())
-        self._sort_descending_cb(None)
+	if self.settings.episodes_order == self.settings.ASCENDING_ORDER:
+		self._sort_ascending_cb(None)
+	else:
+	        self._sort_descending_cb(None)
     
     def _create_menu(self):
         menu = gtk.Menu()
         
-        menuitem= gtk.RadioMenuItem(label = _('A-Z'))
+        menuitem = gtk.RadioMenuItem(label = _('A-Z'))
         menuitem.connect('activate', self._sort_ascending_cb)
         menu.append(menuitem)
+	if self.settings.episodes_order == self.settings.ASCENDING_ORDER:
+		menuitem.set_active(True)
+
         menuitem = gtk.RadioMenuItem(label = _('Z-A'), group = menuitem)
         menuitem.connect('activate', self._sort_descending_cb)
         menu.append(menuitem)
-        
+        if self.settings.episodes_order == self.settings.DESCENDING_ORDER:
+		menuitem.set_active(True)
+
         menuitem = gtk.MenuItem(_('Mark All'))
         menuitem.connect('activate', self._select_all_cb)
         menu.append(menuitem)
@@ -609,9 +716,11 @@ class EpisodesView(hildon.Window):
     
     def _sort_ascending_cb(self, button):
         self.episodes_check_view.sort_ascending()
+        self.settings.episodes_order = self.settings.ASCENDING_ORDER
     
     def _sort_descending_cb(self, button):
         self.episodes_check_view.sort_descending()
+        self.settings.episodes_order = self.settings.DESCENDING_ORDER
 
 class EpisodesCheckView(gtk.TreeView):
     
@@ -698,7 +807,6 @@ class EpisodeView(hildon.Window):
         menu = gtk.Menu()
         
         menuitem = gtk.MenuItem(_('Edit Info'))
-        menuitem.set_sensitive(False)
         menuitem.connect('activate', self._edit_episode_cb)
         menu.append(menuitem)
         
@@ -772,7 +880,9 @@ class InfoTextView(gtk.TextView):
         if not contents:
             return
         if label:
-            contents = _('\n%s: %s') % (label, contents)
+            contents = _('\n%(label)s: %(contents)s') % {'label': label,
+                                                         'contents': contents,
+                                                        }
         self.get_buffer().insert(self.iter, contents)
     
     def clear(self):
@@ -792,11 +902,10 @@ class NewShowsDialog(gtk.Dialog):
         self.set_title(_('Add Shows'))
         contents = gtk.HBox(True, 0)
         self.search_shows_button = gtk.Button()
-        self.search_shows_button.set_label(_('Search Series'))
+        self.search_shows_button.set_label(_('Search Shows'))
         self.search_shows_button.connect('clicked', self._button_clicked_cb)        
         self.manual_add_button = gtk.Button()
         self.manual_add_button.set_label(_('Add Manually'))
-        self.manual_add_button.set_sensitive(False)
         self.manual_add_button.connect('clicked', self._button_clicked_cb)
         contents.add(self.search_shows_button)
         contents.add(self.manual_add_button)
